# Wednesday, December 5th
# functions to fit increasing single index model LSE in lambda-weighted least squares
# the two main functions are find.mle.GoF and search.mle.GoF
# find.mle.GoF is exact but only works in 2D
# search.mle.GoF implements a stochastic search and works for any dimension (matches results from 2D)

library(Iso)
library(rgl)
library(movMF)
library(np)

find.loglik.GoF	<-	function(mle, y, nn, lambda){
	
	ll			<-	-sum(lambda*(y-mle)^2)
	return(ll)

}

find.pMLE.GoF	<- function(alpha, y, x, nn, lambda){	

	delta			<-	x %*% alpha
	tt				<-	order(delta)
	mle				<-	pava(y[tt],nn[tt])
	ll				<- 	find.loglik.GoF(mle, y[tt], nn[tt], lambda[tt])

	return(list(ll=ll, ord=tt, mle=mle))
}

tau			<-	function(r){
	
	k	<-	length(r)
	tau	<-	0
	for(i in 1:(k-1)){
		tau	<-	tau+sum(r[i]>r[(i+1):k])		
	}
	
	return(tau)
	
}

my.inv	<-	function(z){
	
	if((z[1]>=0)*(z[2]>=0)) {theta <- asin(z[2])}
	if((z[1]<0)*(z[2]>=0)) {theta <- acos(z[1])}
	if((z[1]<0)*(z[2]<0)) {theta <- 2*pi-acos(z[1])}
	if((z[1]>=0)*(z[2]<0)) {theta <- 2*pi+asin(z[2])}

	return(theta)	
}

getkmax		<-	function(x, k){
	
	y		<-	sort(x, decreasing=TRUE)
	kmax		<-	y[k]
	locs		<-	which(x >= kmax)
	
	return(locs)
}
# ---------------------------

search.mle.GoF		<-	function(x, y, nn, lambda, B=10000, k=100, kappa0=100, tol=1e-10, max.iter=20, print=TRUE, doplot=FALSE){
	
	m			<-	length(x[,1])
	d			<-	length(x[1,])
	
	
	if(d!=3)	doplot	<-	FALSE


	alpha		<-	matrix(rnorm(d*B), B, d)
	alpha		<-	alpha/apply(alpha,1, function(x) sqrt(sum(x^2)))

	if(((doplot==TRUE)*(d==3))==1){
	plot3d(alpha, xlim=c(-1,1), ylim=c(-1,1), zlim=c(-1,1))
	}

	loglik		<-	rep(0, B)
	for(i in 1:B){
		res				<-	find.pMLE.GoF(alpha[i,], y , x, nn, lambda)	
		loglik[i]		<-	res$ll			
		}
	loc			<-	which.max(loglik)
	alphahat		<-	alpha[loc]
	llhat		<-	loglik[loc]

	check		<-	10

	for(j in 1:max.iter){
	
		if(print==TRUE) print(check)	
		if(check < tol) break	
	
		if(print==TRUE) print(j)

		locs		<-	getkmax(loglik, k)
		if(((doplot==TRUE)*(d==3))==1){
		plot3d(alpha[locs,], xlim=c(-1,1), ylim=c(-1,1), zlim=c(-1,1))
		}

		BB			<-	B/k
		alpha2		<-	matrix(0,BB*k, d)
		kappa		<-	kappa0*(j^2)

		for(i in 1:k){

			mm						<-	alpha[locs[i],]
			temp					<-	rmovMF(BB-1, theta=kappa*mm)
			alpha2[(i-1)*BB+1:BB,]	<-	rbind(mm, temp)
			
		}

		if(((doplot==TRUE)*(d==3))==1){
		plot3d(alpha2, xlim=c(-1,1), ylim=c(-1,1), zlim=c(-1,1), alpha=0.1)
		plot3d(alpha[locs,], xlim=c(-1,1), ylim=c(-1,1), zlim=c(-1,1), add=TRUE, col="red")
		}

		loglik2		<-	rep(0, B)
		for(i in 1:B){
			res				<-	find.pMLE.GoF(alpha2[i,], y , x, nn, lambda)	
			loglik2[i]		<-	res$ll			
			}
		llhat2		<-	max(loglik2)
		check		<-	(llhat2-llhat)/abs(llhat)

		if(print==TRUE) print(c(llhat, llhat2, check))

		loglik		<-	loglik2
		alpha		<-	alpha2
		llhat		<-	llhat2

		}

	loc			<-	which(loglik==max(loglik))
	alphahat	<-	alpha[loc,]

	if(length(loc)>1)	alphahat1	<-	alphahat[1,]
	else		alphahat1	<-	alphahat
	mle			<-	find.pMLE.GoF(alphahat1, y , x, nn, lambda)	
	zhat		<-	x %*% alphahat1

#	return(list(alphahat=alphahat, mle=mle$mle, zhat=sort(as.vector(zhat))), ll=max(loglik))
	return(list(ll=max(loglik)))		
}

find.mle.GoF	<-	function(x, y, nn, lambda){
	
	m		<-	length(x[,1])

	theta	<-	matrix(NA,m,m)
	for(i in 1:(m-1)){
		for(j in (i+1):m){
			d			<-	x[i,]-x[j,]
			d			<-	d/sqrt(sum(d^2))
			z			<-	c(-d[2], d[1])
			theta[i,j]	<-	my.inv(z)
			}	
	}
	
	phi		<-	as.vector(theta)
	phi		<-	phi[!is.na(phi)]
	phi		<-	c(phi, (phi+pi) %% (2*pi))
	phi		<-	sort(phi)
	phi		<-	unique(phi)
	
	beta		<-	phi+c(phi[2:length(phi)], phi[1]+2*pi)
	beta		<-	(beta/2) %% (2*pi)

	loglik	<-	rep(0, length(beta))

	for(i in 1:length(beta)){
		alpha		<-	c(cos(beta[i]), sin(beta[i]))
		res			<-	find.pMLE.GoF(alpha, y , x, nn, lambda)	
		loglik[i]	<-	res$ll	
	}

	loc			<-	which(loglik==max(loglik))
	thetahat	<-	beta[loc]
	thetahat	<-	thetahat %% (2*pi)
	alphahat	<-	matrix(c(cos(thetahat), sin(thetahat)), ncol=2)
	
 	phat	<-	matrix(0, nrow=length(loc), m)
 	ordhat	<-	matrix(0, nrow=length(loc), m)
# 	yordhat	<-	matrix(0, nrow=length(loc), m)
 	for(i in 1:length(loc)){
		alpha		<-	alphahat[i,]
		mle			<-	find.pMLE.GoF(alpha, y , x, nn, lambda)
		phat[i,]	<-	mle$mle	
 		ordhat[i,]	<-	mle$ord	
# 		yordhat[i,]	<-	y[mle$ord]
 	}
#	phat	<-	unique(phat)
#	phat	<-	as.vector(unique(phat))
	
	return(list(phat=phat, alphahat=alphahat, thetahat=thetahat, ord=ordhat, beta=beta, ll=max(loglik)))
}



