# Thursday, February 21st
# R code for data analysis to match Section 4.2 in Manuscript

# read in functions/packages
source("SIMfast.R")
source("SIMfast_GoF.R")
library("rgl")

# read in data 
data		<-	read.table("data.txt", header=TRUE)
xx		<-	t(rbind(data$MMS/10, data$PMA))
yy		<-	data$dead/data$total
nn		<-	data$total


# ---------------------------- #
# ISOTONIC SINGLE INDEX MODEL WITH LINEAR CONTOURS (MODEL 3)
# ---------------------------- #


model3			<-	find.mle(xx, yy, nn)
# fitted value of alpha
model3$alphahat
# fitted values of mu_0
model3$phat


# ---------------------------- #
# ISOTONIC SINGLE INDEX MODEL WITH ELLIPTICAL CONTOURS (MODEL 4)
# ---------------------------- #

set.seed(1773119)

xx4				<-	cbind(xx, xx[,1]^2, xx[,1]*xx[,2], xx[,2]^2)
model4			<-	search.mle(xx4, yy, nn, print=FALSE)
# fitted value of alpha
model4$alphahat
# fitted values of mu_0
model4$mle



# ---------------------------- #
# ISOTONIC SINGLE INDEX MODEL WITH ELLIPTICAL CONTOURS (parallel to axes, MODEL 5)
# ---------------------------- #

set.seed(718655)
xx5				<-	cbind(xx, xx[,1]^2,  xx[,2]^2)
model5			<-	search.mle(xx5, yy, nn, print=FALSE)
# fitted value of alpha
model5$alphahat
# fitted values of mu_0
model5$mle



# ------------------------------------------ #
# GOODNESS OF FIT for MODEL 3 
# ------------------------------------------ #

# estimate quantile of hat D^2_n

xx
yy
nn
lambda	<-	nn/sum(nn)
m		<-	length(nn)

sim.mle	<-	find.mle(xx, yy, nn)
muhat	<-	model3$phat
lprime	<-	1/(yy[model3$ord]*(1-yy[model3$ord]))
test		<-	sum(nn)*sum(lprime*lambda[model3$ord]*((yy[model3$ord]-muhat)^2))

#seed	<-round(10000000*runif(1))
#seed
set.seed(3297472)

B	<-	1000
rss	<-	rep(0,B)

for(i in 1:B){

print(i)	
zz		<-	rnorm(m)/sqrt(lambda)
gof		<-	find.mle.GoF(xx, zz, nn, lambda)
rss[i]	<-	-gof$ll
		
}

#hist(rss)

q		<-	quantile(rss, 0.95)
test>q
test
#16.58547
q
#16.65696 
# estimated pvalue
mean(rss>test)
# 0.051


# therefore, we fail to reject the null hypothesis


# ------------------------------------------ #
# GOODNESS OF FIT for MODEL 4 
# ------------------------------------------ #

# estimate quantile of hat D^2_n

set.seed(1773119)

xx
yy
nn
lambda	<-	nn/sum(nn)
m		<-	length(nn)
xx4		<-	cbind(xx, xx[,1]^2, xx[,1]*xx[,2], xx[,2]^2)

model4	<-	search.mle(xx4, yy, nn, print=FALSE)
muhat	<-	model4$mle
ord		<-	order(xx4 %*% model4$alphahat[1,])
lprime	<-	1/(yy[ord]*(1-yy[ord]))
test	<-	sum(nn)*sum(lprime*lambda[ord]*((yy[ord]-muhat)^2))


B	<-	1000
rss	<-	rep(0,B)

for(i in 1:B){
	
print(i)
zz		<-	rnorm(m)/sqrt(lambda)
gof		<-	search.mle.GoF(xx4, zz, nn, lambda, print=FALSE)
rss[i]	<-	-gof$ll

		
}

#hist(rss)

q		<-	quantile(rss, 0.95)
test>q
test
# 3.033341
q
# 9.88051
# estimated p-value
mean(rss>test)
# 0.777

# therefore, we fail to reject the null hypothesis


# ------------------------------------------ #
# GOODNESS OF FIT for MODEL 4 
# ------------------------------------------ #

# estimate quantile of hat D^2_n

set.seed(718655)

xx
yy
nn
lambda	<-	nn/sum(nn)
m		<-	length(nn)
xx5		<-	cbind(xx, xx[,1]^2, xx[,2]^2)

model5	<-	search.mle(xx5, yy, nn, print=FALSE)
muhat	<-	model5$mle
ord		<-	order(xx5 %*% model5$alphahat[1,])
lprime	<-	1/(yy[ord]*(1-yy[ord]))
test	<-	sum(nn)*sum(lprime*lambda[ord]*((yy[ord]-muhat)^2))


B	<-	1000
rss	<-	rep(0,B)

for(i in 1:B){
	
print(i)
zz		<-	rnorm(m)/sqrt(lambda)
gof		<-	search.mle.GoF(xx4, zz, nn, lambda, print=FALSE)
rss[i]	<-	-gof$ll

		
}

#hist(rss)

q		<-	quantile(rss, 0.95)
test>q
test
# 3.033341
q
# 10.01583
# estimated pvalue
mean(rss>test)
# 0.778

# therefore, we fail to reject the null hypothesis
